package com.yixin.ms.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.UUID;
import cn.hutool.core.util.DesensitizedUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.excel.util.CollectionUtils;
import com.alibaba.excel.util.DateUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.beust.jcommander.internal.Lists;
import com.yixin.ms.boot.config.QrcodeApiConfig;
import com.yixin.ms.boot.core.ApiPath;
import com.yixin.ms.boot.restful.RestCode;
import com.yixin.ms.boot.restful.RestResponse;
import com.yixin.ms.boot.restful.RestResult;
import com.yixin.ms.boot.restful.ServiceException;
import com.yixin.ms.boot.uitls.DESUtil;
import com.yixin.ms.boot.uitls.OkHttpUtils;
import com.yixin.ms.boot.uitls.RedisUtil;
import com.yixin.ms.boot.uitls.SMSOrIdCardUtils;
import com.yixin.ms.mapper.PersonInfoMapper;
import com.yixin.ms.model.dto.*;
import com.yixin.ms.model.dto.HealthCodeDTO;
import com.yixin.ms.model.entity.PersonInfo;
import com.yixin.ms.model.enums.*;
import com.yixin.ms.model.vo.AdditionalRQMT;
import com.yixin.ms.model.vo.HealtCodeNucleinVO;
import com.yixin.ms.model.vo.HealtCodeTypeVO;
import com.yixin.ms.model.vo.HealtCodeUserVO;
import com.yixin.ms.server.*;
import com.yixin.ms.server.dto.*;
import com.yixin.ms.server.vo.*;
import com.yixin.ms.service.HealthCodeService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

@Slf4j
@Service
public class HealthCodeServiceImpl implements HealthCodeService {
    @Autowired
    private RedisUtil redisUtil;
    @Autowired
    private PlaceHealthCodeServer placeHealthCodeServer;
    @Autowired
    private HealthCodeServer healthCodeServer;
    @Autowired
    private GuizhouHealthServer guizhouHealthServer;
    @Resource
    private UserBlackServer userBlackServer;

    @Autowired
    private QrcodeApiConfig qrcodeApiConfig;

    @Resource
    private PersonInfoMapper personInfoMapper;

    @Override
    public HealtCodeUserVO getShortTermCertificate(ShortTermDTO shortTermDTO) {
        try {
            HealtCodeUserVO shortTermUser = getShortTermUser(shortTermDTO.getIdNumber(), shortTermDTO.getName());
            if (shortTermUser != null && !CollectionUtils.isEmpty(shortTermUser.getData())) {
                return shortTermUser;
            }
            return getSjJkm(shortTermDTO.getIdNumber(), shortTermDTO.getName());
        } catch (Exception e) {
            e.printStackTrace();
            log.info("异常 :" + e.getMessage());
            try {
                return getSjJkm(shortTermDTO.getIdNumber(), shortTermDTO.getName());
            } catch (Exception ex) {
                ex.printStackTrace();
                throw new ServiceException("未获取到信息，请联系工作人员");
            }
        }

    }

    @Override
    public String addToken(String token) {
        String s = "hh_token_" + UUID.randomUUID();
        boolean hanghui = redisUtil.set(s, s);
        if (hanghui) {
            return s;
        } else {
            throw new ServiceException("体检失败");
        }

    }

    @Override
    public Boolean deleteToken(String token) {
        return redisUtil.delete(token);
    }

    @Override
    public RestResult getShortTerm(ShortTermDTO shortTermDTO) {
        log.info("请求参数:{}", JSON.toJSONString(shortTermDTO));
        if (StringUtils.isBlank(shortTermDTO.getIdNumber()) && ObjectUtil.isEmpty(shortTermDTO.getHealthCode())) {
            throw new ServiceException(701, "身份证或二维码不能为空");
        }

        HealtCodeUserVO userVO = new HealtCodeUserVO();

        // 20230309新增：添加额外返回参数
        Boolean ifNeedXinghuoApi = placeHealthCodeServer.getIfNeedXinghuoApiBySn(shortTermDTO.getSn());
        // 设备开启星火服务且 有二维码 没身份证
        if (ifNeedXinghuoApi
                && ObjectUtil.isNotEmpty(shortTermDTO.getHealthCode())
                && ObjectUtil.isEmpty(shortTermDTO.getIdNumber())) {
            PersonInfo personInfo = putAdditionalRQMT(userVO, shortTermDTO);
            if (ObjectUtil.isEmpty(shortTermDTO.getIdNumber())) {
                shortTermDTO.setIdNumber(personInfo.getIdCardNo());
                shortTermDTO.setName(personInfo.getTrueName());
            }
            userVO.setPhone(personInfo.getPhone());
        }

        initHealthInfo(userVO);

        //姓名
        userVO.setUsername(StrUtil.isEmpty(userVO.getUsername()) ? shortTermDTO.getName() : userVO.getUsername());
        //身份证号
        userVO.setIdNumber(StrUtil.isEmpty(userVO.getIdNumber()) ? shortTermDTO.getIdNumber() : userVO.getIdNumber());
        userVO.setShowIdNumber(StrUtil.isEmpty(userVO.getIdNumber()) ? StrUtil.EMPTY : DesensitizedUtil.idCardNum(userVO.getIdNumber(), 4, 4));
        userVO.setShowUsername(StrUtil.isEmpty(userVO.getUsername()) ? StrUtil.EMPTY : DesensitizedUtil.chineseName(userVO.getUsername()));
        //用户健康码信息
        HealtCodeTypeVO codeTypeVO = userVO.getData().stream().findFirst().get();
        //用户核酸信息
        List<HealtCodeNucleinVO> nucleinList = userVO.getNuclein();
        if (CollectionUtil.isNotEmpty(nucleinList)) {
            nucleinList.forEach(v -> v.setShowTime(v.getReportTime() == null ? v.getChecktime() : DateUtil.compare(v.getChecktime(), v.getReportTime()) > 0 ? v.getChecktime() : v.getReportTime()));
        }

//        if (HealthCodeLevelEnum.GREEN.getDesc().equals(codeTypeVO.getMzt())) {
//            codeTypeVO.setContent("绿码：健康码状态为低风险").setContentColor(ColorEnum.GREEN.getValue());
//            userVO.setBackgroundColor(ColorEnum.GREEN.getValue());
//        }
//        userVO.setIsOpen(HealthOpenEnum.RELEASE.getCode()).setOpenMsg("绿码，请通行").setOpenVoiceMsg("绿码，请通行");

//        userVO.setNuclein(Lists.newArrayList(new HealtCodeNucleinVO().setTitle("核酸检测").setIconUrl(NucleicStatusEnum.NOT_AVAILABLE.getUrl()).setContent("暂无").setContentColor(ColorEnum.GREEN.getValue())));


//        userVO.setTitle("健 康 码").setTitleColor(ColorEnum.WHITE.getValue())
//                //疫苗信息
//                .setVaccineIcoTitle("新冠疫苗").setVaccineIconUrl(HealthVaccineNumEnum.numOf(userVO.getVaccineNum()).getUrl()).setVaccineNumColor(ColorEnum.GREEN.getValue())
//                //行程信息
//                .setTravelValidationDesc(TravelValidationEnum.codeOf(userVO.getTravelValidation()).getDesc()).setTravelValidationTitle("动态行程卡").setTravelValidationIconUrl(TravelValidationEnum.codeOf(userVO.getTravelValidation()).getUrl());

        return RestResponse.ok(userVO);
    }

    private void initHealthInfo(HealtCodeUserVO userVO) {
        HealtCodeNucleinVO v = new HealtCodeNucleinVO();
        userVO.setIsOpen(HealthOpenEnum.RELEASE.getCode());
        userVO.setOpenMsg("请通行");
        userVO.setOpenVoiceMsg("请通行");

        v.setShowTime(null);
        Date date = null;
        String format = DateUtils.format(new Date(), DateUtils.DATE_FORMAT_19);
        try {
            date = DateUtils.parseDate(format, DateUtils.DATE_FORMAT_19);
        } catch (Exception e) {
            e.printStackTrace();
        }
        v.setChecktime(date);
        v.setReportTime(date);
        v.setContent("佩戴口罩").setContentColor(ColorEnum.GREEN.getValue());
        v.setIconUrl("https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/green_tick.png");
        v.setTitle("温馨提醒");
        v.setResult("阴性");
        userVO.setNuclein(Arrays.asList(v));

        HealtCodeTypeVO codeTypeVO = new HealtCodeTypeVO();
        codeTypeVO.setMzt("绿码");
        codeTypeVO.setContent("请通行");
        codeTypeVO.setContentColor(ColorEnum.GREEN.getValue());
        userVO.setData(Arrays.asList(codeTypeVO));

        userVO.setVaccineNum("勤洗手");
        String title = "龙洞堡机场";
        userVO.setTitle(title == null ? "新冠疫苗" : title);
        userVO.setTitleColor(ColorEnum.WHITE.getValue());
        userVO.setVaccineIcoTitle("温馨提醒");
        userVO.setVaccineIconUrl("https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/green_tick.png");
        userVO.setVaccineNumColor(ColorEnum.GREEN.getValue());
        userVO.setTravelValidationTitle("动态行程卡");
        userVO.setTravelValidationIconUrl("https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/trip/green.png");
        userVO.setTravelValidationDesc(null);
        userVO.setBackgroundColor(ColorEnum.GREEN.getValue());
    }

    /**
     * 20230309新增：添加额外返回参数
     *
     * @param userVO HealtCodeUserVO
     */
    private PersonInfo putAdditionalRQMT(HealtCodeUserVO userVO, ShortTermDTO shortTermDTO) {
        GetProofByQrCodeDto getProofByQrCodeDto = new GetProofByQrCodeDto();
        getProofByQrCodeDto.setQrCodeText(shortTermDTO.getHealthCode());
        getProofByQrCodeDto.setMacAddress(shortTermDTO.getMacAddress());
        String result = HttpUtil.post(qrcodeApiConfig.getBaseUrl() + ApiPath.GET_PROOF_BY_QRCODE.path, JSON.toJSONString(getProofByQrCodeDto), 20 * 1000);
        log.info("星火服务返回：" + result);
        JSONObject resultJson = JSON.parseObject(result);
        String errcode = resultJson.getString("errcode");
        String errmsg = resultJson.getString("errmsg");
        if (!"0".equals(errcode)) {
            throw new ServiceException(errmsg);
        }
        GetProofByQrCodeApiResult data = resultJson.getObject("data", GetProofByQrCodeApiResult.class);
        PersonInfo personInfo = new PersonInfo();
        if (ObjectUtil.isNotEmpty(data)) {
            AdditionalRQMT additionalRQMT = new AdditionalRQMT();
            additionalRQMT.setPhoto(data.getPhoto());
            additionalRQMT.setPhotoType("base64");
            personInfo.setIdCardNo(data.getIdCardNo());
            personInfo.setTrueName(data.getName());
            personInfo.setAddress(data.getAddress());
            personInfo.setNation(data.getNation());
            personInfo.setPhone(data.getPhone());
            personInfo.setApplyTime(data.getApplyTime());
            personInfo.setValidTime(data.getValidTime());
            personInfo.setSex(data.getSex());
            personInfo.setSn(shortTermDTO.getSn());
            personInfoMapper.insert(personInfo);
            additionalRQMT.setPersonId(personInfo.getId().toString());
            additionalRQMT.setNeedCompare(true);
            userVO.setAdditionalRQMT(additionalRQMT);
        }
        return personInfo;
    }

    /**
     * 根据barCode查询健康码信息
     *
     * @param shortTermDTO
     * @return
     */
    private HealthResultVO<DeviceScanIdCardVO> deviceScanCode(ShortTermDTO shortTermDTO) {
        //健康码查询场所码信息
        HealthResultVO<DeviceScanIdCardVO> scanIdCardResult = placeHealthCodeServer.deviceScanCode(new DeviceScanCodeDTO().setDeviceId(shortTermDTO.getSn()).setHealthCodeQrCode(shortTermDTO.getHealthCode()).setUrlParams(shortTermDTO.getHealthCode()));
        //换绑场所码或者管理员健康码首次会提示设备未打卡等错误，重新发起调用
        if (scanIdCardResult == null || !scanIdCardResult.getCode().equals(Integer.valueOf(RestCode.SUCCESS.getCode()))) {
            if (scanIdCardResult != null && scanIdCardResult.getCode().equals(Integer.valueOf(RestCode.INVALID_TOKEN.getCode()))) {
                //清理token缓存
                placeHealthCodeServer.getToken(true);
            }

            try {
                Thread.sleep(500);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            scanIdCardResult = placeHealthCodeServer.deviceScanCode(new DeviceScanCodeDTO().setDeviceId(shortTermDTO.getSn()).setHealthCodeQrCode(shortTermDTO.getHealthCode()).setUrlParams(shortTermDTO.getHealthCode()));
        }

        return scanIdCardResult;
    }

    /**
     * 根据证件号查询健康码信息
     *
     * @param shortTermDTO
     * @return
     */
    private HealthResultVO<DeviceScanIdCardVO> deviceScanIdCardEncrypt(ShortTermDTO shortTermDTO) {
        DeviceScanIdCardDTO scanIdCardDTO = new DeviceScanIdCardDTO().setDeviceId(shortTermDTO.getSn()).setIdNumber(shortTermDTO.getIdNumber());
        if (SMSOrIdCardUtils.isIdCard(shortTermDTO.getIdNumber())) {
            scanIdCardDTO.setIdType(IDTypeEnum.ID_CARD.getCode());
        } else {
            scanIdCardDTO.setIdType(IDTypeEnum.PASSPORT.getCode());
        }

        //扫人员证件查询场所码信息
        HealthResultVO<DeviceScanIdCardVO> scanIdCardResult = placeHealthCodeServer.deviceScanIdCardEncrypt(scanIdCardDTO);
        //换绑场所码或者管理员健康码首次会提示设备未打卡等错误，重新发起调用
        if (scanIdCardResult == null || !scanIdCardResult.getCode().equals(Integer.valueOf(RestCode.SUCCESS.getCode()))) {
            if (scanIdCardResult != null && scanIdCardResult.getCode().equals(Integer.valueOf(RestCode.INVALID_TOKEN.getCode()))) {
                //清理token缓存
                placeHealthCodeServer.getToken(true);
            }

            try {
                Thread.sleep(500);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            scanIdCardResult = placeHealthCodeServer.deviceScanIdCardEncrypt(scanIdCardDTO);
        }

        return scanIdCardResult;
    }

    @Override
    public RestResult<HealthCodeVO> queryInfoByBarCode(HealthCodeDTO healthCodeDTO) {
        IrsTripAuthEnum authEnum = IrsTripAuthEnum.appKeyOf(healthCodeDTO.getAppKey());
        if (authEnum == null || !authEnum.getAppSecret().equals(healthCodeDTO.getAppSecret())) {
            return RestResponse.error("调用权限校验失败，请检查");
        }

        return healthCodeServer.queryInfoByBarCode(healthCodeDTO.getBarCode());
    }

    /**
     * 人脸比对结果获取
     *
     * @param comparionResultDto ComparionResultDto
     */
    @Override
    public void comparionResult(ComparionResultDto comparionResultDto) {
        PersonInfo personInfo = personInfoMapper.selectById(comparionResultDto.getPersonId());
        if (ObjectUtil.isNotEmpty(personInfo)) {
            personInfo.setComparedState(1);
            personInfoMapper.updateById(personInfo);
        }
    }

    public HealtCodeUserVO getShortTermUser(String sfzh, String name) throws Exception {
        log.info("江干分局健康码传参，身份证{} 姓名 {}", sfzh, name);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("sfzh", sfzh);
        jsonObject.put("xm", name);
        String sync = OkHttpUtils.builder().url("http://115.236.38.205:8903/id/hvn")
                .addHeader("Content-Type", "application/json; charset=utf-8")
                .localPost(jsonObject.toJSONString())
                .sync();
        if (sync.contains("请求失败：Connect timed out")) {
            throw new ServiceException("江干分局请重试");
        }
        log.info("江干分局健康码返回结果，{}", sync);
        HealtCodeUserVO healtCodeUserVO = new HealtCodeUserVO();
        if (StringUtils.isNotBlank(sync)) {
            JSONObject json = JSON.parseObject(sync);
            if (StringUtils.equals("00", json.getString("error_code"))) {
                healtCodeUserVO.setData(JSON.parseArray(json.getJSONArray("data").toJSONString(), HealtCodeTypeVO.class));
                healtCodeUserVO.setNuclein(JSON.parseArray(json.getJSONArray("nuclein").toJSONString(), HealtCodeNucleinVO.class));
                //源数据只返回报告时间，检测时间先等同于报告时间，方便后面逻辑处理
//                healtCodeUserVO.getNuclein().forEach(v -> v.setReportTime(v.getChecktime()));
                healtCodeUserVO.setVaccineNum(json.getString("vaccine_num"));
            } else {
                throw new ServiceException("江干分局异常:" + jsonObject.getString("msg"));
            }
        }
        return healtCodeUserVO;
    }

    public HealtCodeUserVO getSjJkm(String sfzh, String name) throws Exception {
        log.info("市局健康码传参，身份证{} 姓名 {}", sfzh, name);
        String token = "";
        token = (String) redisUtil.get("jkm-token");
        if (StringUtils.isBlank(token)) {
            token = getToken();
            if (StringUtils.isBlank(token)) {
                throw new ServiceException("市局健康码请重试");
            }
        }
        JSONObject data = new JSONObject();
        JSONObject body = new JSONObject();
        body.put("idCard", sfzh);
        String encryptSfzh = DESUtil.encrypt(body.toJSONString(), "7760ce65a8544f779f36607c201e8a0c");
        data.put("body", encryptSfzh);
        data.put("token", token);
        String sync = OkHttpUtils.builder().url("http://policecard.tenzont.com/auth/api/healthCode")
                .addHeader("Content-Type", "application/json; charset=utf-8")
                .localPost(data.toJSONString())
                .sync();
        if (sync.contains("请求失败：Connect timed out")) {
            throw new ServiceException("市局健康码请重试");
        }
        log.info("市局健康码返回结果，{}", sync);
        JSONObject jkm = JSONObject.parseObject(sync);
        String rspCode = jkm.getJSONObject("header").getString("rspCode");
        HealtCodeUserVO healtCodeUserVO = new HealtCodeUserVO();
        if (StringUtils.equals("00", rspCode)) {
            String mzt = jkm.getJSONObject("body").getString("mzt");
            if (StringUtils.isNotBlank(mzt)) {
                List<HealtCodeTypeVO> list = new ArrayList<>();
                HealtCodeTypeVO healtCodeTypeVO = new HealtCodeTypeVO();
                healtCodeTypeVO.setMzt(mzt);
                list.add(healtCodeTypeVO);
                healtCodeUserVO.setData(list);
                healtCodeUserVO.setNuclein(new ArrayList<>());
                healtCodeUserVO.setVaccineNum("");
                return healtCodeUserVO;
            } else {
                throw new ServiceException("市局健康码请重试");
            }
        } else if (StringUtils.equals("01", rspCode)) {
            getToken();
            HealtCodeUserVO sjJkm = getSjJkm(sfzh, name);
            return sjJkm;
        } else {
            throw new ServiceException("市局健康码请重试");
        }
    }


    public String getToken() throws Exception {
        String sync = "";
        JSONObject jsonObject = new JSONObject();
        JSONObject body = new JSONObject();
        body.put("appid", "jzz3c086d786420a958a143e6154a351");
        body.put("secret", "ecbe0e220ec74d05b2b22268a737d2f1");
        jsonObject.put("body", body);
        try {
            sync = OkHttpUtils
                    .builder()
                    .url("http://policecard.tenzont.com/auth/api/token")
                    .localPost(jsonObject.toJSONString())
                    .sync();
        } catch (Exception e) {
            log.info("获取健康码的token接口异常 ，{}", e.getMessage());
            return "";
        }
        if (StringUtils.isBlank(sync)) {
            return "";
        }
        JSONObject fhz = JSONObject.parseObject(sync);
        String rspCode = fhz.getJSONObject("header").getString("rspCode");
        if (StringUtils.equals("00", rspCode)) {
            String token = fhz.getJSONObject("body").getString("token");
            if (StringUtils.isNotBlank(token)) {
                redisUtil.set("jkm-token", token, 7200);
                return token;
            }
        }
        return "";
    }

//    public static void main(String[] args) throws Exception {
//        System.out.println(getSjJkm("412728199706030513", ""));
//    }
}

